/*
 * FIRA2 is a program to inject/hide a file into a standard bitmap file. 
 * It includes file compression and encryption as well.
 * Copyright (C) 2006 LCC Coprs
 * The author can be reached at aurora.lai@gmail.com
 *
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


/* 
 * FILE: FileAccess.java
 * INHERITS: RandomAccessFile
 * PURPOSE: serve as I/O file object than can be injected with byte value and 
 *          injected byte value can be retrieved from
 * AUTHOR: LCC Corps
 * DATE: 26 November 2005
 * LAST UPDATED: June 16, 2006
 * REFERENCE:
 * COMMENT:
 *
 */


package sys;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.RandomAccessFile;
import java.io.IOException;


public class FileAccess extends RandomAccessFile
{
  // Number of bit to be injected into one byte. Must be power of 2, i.e. 1, 2, 4, 8.
  public static final int BIT_PER_BYTE = 2;     

  // CONSTRUCTOR #1: MyFile ()
  // IMPORT: file to be injected with text, file read/write mode as required 
  //         by RandomAccessFile constructor
  // EXPORT: none
  // COMMENT: throws FileNotFoundException

  public FileAccess (File file, String mode)
  throws FileNotFoundException
  {
    super (file, mode);
  }


  // CONSTRUCTOR #2: MyFile ()
  // IMPORT: file to be injected with text, file read/write mode as required
  //         by RandomAccessFile constructor
  // EXPORT: none
  // COMMENT: throws FileNotFoundException

  public FileAccess (String pathname, String mode)
  throws FileNotFoundException
  {
    super (pathname, mode);
  }


  // METHOD: injectByte ()
  // PURPOSE: to inject a byte to a bitmap file
  // IMPORT: a byte value to be injected, location of injection in the target file
  // EXPORT: none
  // COMMENT: Injection process will start from right to left. Due to the fact 
  //		  that Intel micro processor store data in Big Endian, the least
  //		  significant bit (LSB) will be inject first, followed by the most
  //		  significant bit.
  //
  //   e.g.  a byte      7 6 5 4  |  3 2 1 0
  //					   (MSB)      (LSB)
  //
  //         injected    xxxx xxx0
  //                     xxxx xxx1
  //                     xxxx xxx2
  //                     xxxx xxx3
  //                     xxxx xxx4
  //                     xxxx xxx5
  //                     xxxx xxx6
  //                     xxxx xxx7
  //

  public void injectByte (byte value, long pos)
  {
    int temp;
    byte mask = 0;
    
    for (int i = 0; i < BIT_PER_BYTE; i++) {
        mask <<= 1;
        mask |= 0x01;
    }

    for (int i = 0; i < 8/BIT_PER_BYTE; i++) {
      try {
        seek (pos);           // move the file pointer to the desired position
        temp = readByte ();   // get the byte of the targeted file to be modified
        temp &= ~mask;         // clear the last bit (to be modified)

        temp |= value & mask;   // OR the targeted byte with one of the text bit
                                // (to be injected)
        value >>>= BIT_PER_BYTE;   // shift the text to be injected

        seek (pos++);
        writeByte (temp);
      } catch (IOException e) {
        System.err.println ("ERROR: Unable to inject value at position " +pos);
      }
    }
  }


  // METHOD: retrieveByte ()
  // PURPOSE: to retrieve a byte form a bitmap file with injected byte
  // IMPORT: location in the target file, where a byte is to be retrieved
  // EXPORT: a retrieved byte value
  // COMMENT: Retrieval process, is the opposite of the injection process. 
  //          LSB will be retrieved first, MSB will be the last.

  public byte retrieveByte (long pos)
  {
    byte retrieved = 0x00;
    byte temp;
    byte mask = 0;
    
    for (int i = 0; i < BIT_PER_BYTE; i++) {
        mask <<= 1;
        mask |= 0x01;
    }

    try {
      seek (pos);   // move the file pointer to desired position

      // retrieve a byte, starting with LSB, ends with MSB
      for (int i = 0; i < 8/BIT_PER_BYTE; i++) {
        temp = readByte ();
        retrieved |= (temp & mask) << i*BIT_PER_BYTE;
      }
    } catch (IOException e) {
      System.err.println ("ERROR: Unable to retrieve value at position " +pos);
    }

    return retrieved;    
  }


  // METHOD: eof ()
  // PURPOSE: to return true if the file pointer has reached the end of file.
  // IMPORT: none
  // EXPORT: true if end-of-file encountered

  public boolean eof ()
  {
    try {
      return (getFilePointer () >= length ());
    } catch (IOException e) {
      return true;
    }
  }

}