/*
 * FIRA2 is a program to inject/hide a file into a standard bitmap file. 
 * It includes file compression and encryption as well.
 * Copyright (C) 2006 LCC Coprs
 * The author can be reached at aurora.lai@gmail.com
 *
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


/* 
 * File: FileCompression.java
 * Purpose: to try to implement compression/decompression using Java 
 *          ZipInputStream and ZipOutputStream class
 * Author: LCC Corps
 * Date: April 26, 2006
 * Note: This FileCompression.java is modified slightly from a stand-alone 
 *       file zipping/unzipping application written by the author when learning 
 *       the Java Api in file zipping/unzipping. 
 */

package sys;


import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;
import java.util.zip.ZipEntry;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;


public class FileCompression
{
    // Buffer size used during reading and writing to file
    private static final int BUFFER = 1024;
	
    // File to be compressed or decompressed
    private MyFile readIn, writeOut;
	
    // Constructor #1
    public FileCompression (MyFile source, MyFile dest)
    {
        setReadInFile (source);
        setWriteOutDir (dest);
    }
	
    // Constructor #2
    public FileCompression (String sourceFile, String destFile)
    {
        this (new MyFile (sourceFile), new MyFile (destFile));
    }
	
	
    // Method: unzip()
    // Purpose: to unzip the file
    // Note: This method DOES NOT perform the actual file decompression task.
    //       It only define all the required input and output stream as well as
    //       the required ZipEntry object. It also determine (from the ZipEntry 
    //       object) whether to decompress a file or make new directory as required
    // Import: none
    // Export: none
    // Comment: For actual file decompression, refer to extractFile ()
    public void unzip ()
    {
        // only do the decompression if the specified file is NOT null/dne
        if (readIn != null) {
            if (readIn.isFile ()) {
                FileInputStream fis = null;
                ZipInputStream zin = null;
                ZipEntry zEntry = null;
					
                // define the input stream (of compressed data)
                try {
                    fis = new FileInputStream (readIn);
                    zin = new ZipInputStream (fis);
                } catch (FileNotFoundException e) {
                    System.err.println ("ERROR: Cannot find file \"" 
                            +readIn.getName() +"\"");
                }
	
                try {
                // need a loop to continuously take next ZipEntry. This is because 
                // within a zip file, there can be multiple files, and even directories.
                // getNextEntry() will return ZipEntry which only correspond to ONE 
                // compressed file in the zip file.
                //
                // In short, EACH ZipEntry object correspond to ONE compressed file 
                // in Zip file. Try to remove the loop for reading subsequent ZipEntry, 
                // and all you get is the extraction of ONLY the first compressed 
                // file in the zip file
                    while ((zEntry = zin.getNextEntry ()) != null){
                        MyFile entryDest = new MyFile (writeOut, zEntry.getName ());
                        extractFile (entryDest, zEntry, zin);
                    }
                    
                    // required to close the input stream
                    zin.close ();
                } catch (IOException e) {
                    System.err.println ("ERROR: Error in getting Zip entries...");
                }
            } else {  // if file != null, but the file is a directory, then cannot decompress!
                System.err.println ("ERROR: Unable to decompress..." +
                        "The source must be a file and destination must be a directory...");
            }
        }
    }
	
    // Method: extractFile ()
    // Purpose: to decompress the files in a zip file
    // Note: The actual file decompression is done here
    // Import: destination File object (to which the decompressed file 
    //         is written), current ZipEntry object to be decompressed, 
    //         ZipInputStream object
    // Export: none
    private void extractFile (MyFile dest, ZipEntry entry, ZipInputStream zin)
    {
        int count;
        byte[] temp = new byte[BUFFER];

        try {
            /* if the Directory (into which the decompressed file should be
             * extracted) d.n.e., then create that parrent directory before
             * attempting to write to the destination file (specified by 'fos')
             * to avoid FileNotFoundException.
             * Initial implementation to correct this problem is by 
             * putting the parent directory creating code INSIDE the exception 
             * CATCH block (NOT GOOD).
             */
            MyFile parent = dest.getParentFile ();
	
            if (parent != null && !parent.exists()) {
                parent.mkdirs();
            }
            
            
            FileOutputStream fos = new FileOutputStream (dest);

            // decompressing each ZipEntry until EOF
            while ((count = zin.read (temp, 0, BUFFER)) >= 0) {
                fos.write (temp, 0, count);
            }

            // required to close all the output stream
            fos.flush ();
            fos.close ();
        } catch (IOException e) {
            System.err.println ("ERROR: Error while extracting file: " 
                        +entry.getName());
        }
    }
	
	
    // Method: zip()
    // Purpose: to zip a file or a directory
    // Note: This method DOES NOT perform the actual file compression task.
    //       It only define all the required input and output stream. It also 
    //       determine whether the specified File object is a file or a directory.
    // Import: none
    // Export: none
    // Comment: For actual file compression, refer to compressFiles ()
    public void zip ()
    {
        if (readIn != null && writeOut != null) {
            try {
                FileOutputStream fos = new FileOutputStream (writeOut);
                ZipOutputStream zout = new ZipOutputStream (fos);

                // compress all the files in a directory
                compressFile (readIn, zout);
				
                // close the ZipOutputStream only when all compression task is complete
                zout.close ();	
            }catch (IOException e) {
                System.err.println ("ERROR: Cannot find file: " +readIn.getName ());
            }
        }
    }

    // Method: compressFiles ()
    // Purpose: to compress an array of files
    // Note: The actual file compression is done here
    // Import: an array of files to be compressed and ZipOutputStream object
    //         to write the compressed data to file.
    // Export: none
    // Note: Recursion is used to compress list of files within a directory 
    //       within a directory within a directory...
    private void compressFile (File file, ZipOutputStream zout)
    {
        ZipEntry zEntry = null;
        FileInputStream fis = null;

        int count;
        byte temp[] = new byte[BUFFER];

        try {
            // if the current File object is a file, then compress it
            if (file.isFile ()) {
                zEntry = new ZipEntry (file.getName());
                fis = new FileInputStream (file);
			
                zout.putNextEntry (zEntry);
			
                while ((count = fis.read (temp, 0, BUFFER)) >= 0) {
                    zout.write (temp, 0, count);
                }
			
                fis.close ();
            }
        } catch (IOException e) {
            System.err.println ("ERROR: Error while compressing file: " 
                    +zEntry.getName ());
        }
    }
	
	
    // Method: setReadInFile ()
    // Purpose: a setter method to set the file of the Zip1 object. Will only
    //          set the file if the imported File object exists
    // Import: a File object
    // Export: none
    // COMMENT: Since 
    //              File f = new File (""); --> for current folder
    //              f.exists();	--> false
    //      but     f.getAbsoluteFile().exists(); --> true !!!
    //          So, use MyFile object, instead of File object
    public void setReadInFile (MyFile file)
    {
        if (file != null && file.exists())
            readIn = file;
        else
            System.err.println ("ERROR: Cannot find file: " +file.getName());
    }


    // Method: setWriteOutDir ()
    // Purpose: a setter method to set the file of the Zip1 object. Will only
    //          set the file if the imported File object exists
    // Import: a File object
    // Export: none
    // COMMENT: If the writeOut file is constructed with "" and parameter, 
    //          the compressed files will be extracted to the current working directory
    public void setWriteOutDir (MyFile file)
    {
        if (file != null) {
            // due to the ClassCastException, this is other alternative for
            // setting the writeOut file
            writeOut = new MyFile(file.getAbsolutePath ());
        } else {
            System.err.println ("ERROR: Cannot find file: " +file.getName());
        }
    }


    // Method: print ()
    // Purpose: for debugging uses
    public void print (String a)
    {
        System.err.println (a);
    }
	
}