/*
 * FIRA2 is a program to inject/hide a file into a standard bitmap file. 
 * It includes file compression and encryption as well.
 * Copyright (C) 2006 LCC Coprs
 * The author can be reached at aurora.lai@gmail.com
 *
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


/*
 * File: FileCrypto.java
 * Purpose: Enhanced small program to encrypt and decrypt a single file
 * Author: LCC Corps
 * Date: April 29, 2006
 * Comment: perform a simple encryption and decryption using Java 
 * 			cryptography package
 * Note: BIG THANKS to Terence Tan for his supervision
 *
 */
 

package sys;
 
import java.io.FileNotFoundException;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;

// for Exception related to cryptography, e.g. NoSuchAlgorithmException, 
// InvalidKeyException, GeneralSecurityException, etc
import java.security.*;
import java.security.SecureRandom;

 
public class FileCrypto 
{
    private static final int BUFFER = 8;
	
    // The algorithm for encryption/decryption (NOTE: case-sensitive)
    private static final String CRY_ALG = "DES";
    private static final String KG_ALG = "DES";
	
    private FileInputStream fis;
    private FileOutputStream fos;
    private Cipher c;	// The object that actually do the encryption and decryption
    private KeyGenerator kg;	// SecretKey generator object


    // Constructor #1
    public FileCrypto (MyFile fin, MyFile fout)
    {
        setInputStream (fin);
        setOutputStream (fout);
        setCryptoAlgorithm (CRY_ALG);
        setKGAlgorithm (KG_ALG);
    }

    
    /*
     * METHOD: setInputStream()
     * PURPOSE: to set the input file stream for the file encryption/decryption
     * IMPORT: MyFile object for input stream
     * EXPORT: none
     * COMMENT: 
     */
    public void setInputStream (MyFile in)
    {
        try {
            fis = new FileInputStream (in);
        } catch (FileNotFoundException e) {
            System.err.println ("ERROR: File not found: " +in.getName ());
        }
    }

    
    /*
     * METHOD: setOutputStream()
     * PURPOSE: to set the output file stream for the file encryption/decryption
     * IMPORT: MyFile object for the output stream
     * EXPORT: none
     * COMMENT: 
     */
    public void setOutputStream (MyFile out)
    {
        try {
            fos = new FileOutputStream (out);
        } catch (FileNotFoundException e) {
            System.err.println ("ERROR: File not found: " +out.getName ());
        }
    }


    /*
     * METHOD: setCryptoAlgorithm()
     * PURPOSE: to set the cryptography algorithm to be used
     * IMPORT: String representation of the desired algorithm
     * EXPORT: none
     * COMMENT: 
     */
    public void setCryptoAlgorithm (String algorithm)
    {
        // Will throw NoSuchAlgorithmException when the encryption algorithm
        // specified in the parameter does not exist, and 
        //NoSuchPaddingException when the specified padding algorithm d.n.e.
        try {
            c = Cipher.getInstance (algorithm);
        } catch (GeneralSecurityException e) {
            System.err.println ("ERROR: Specified encryption algorithm does not exist!!!");
            System.err.println ("Default algorithm (" +CRY_ALG +") selected...");
        }
    }

    
    /*
     * METHOD: setKGAlgorithm()
     * PURPOSE: to set the Key Generator algorithm to be used
     * IMPORT: String representation of the desired Key Generator algorithm
     * EXPORT: none
     * COMMENT: 
     */
    public void setKGAlgorithm (String algorithm)
    {
        // Will throw exception when the encryption algorithm specified 
        // in the parameter does not exist
        try {
            kg = KeyGenerator.getInstance (algorithm);
        } catch (GeneralSecurityException e) {
            System.err.println ("ERROR: Specified encryption algorithm does not exist!!!");
            System.err.println ("Default algorithm (" +KG_ALG +") selected...");
        }
    }
	

    /*
     * METHOD: doCrypto()
     * PURPOSE: to actually encrypt/decrypt a file
     * IMPORT: a password in array of bytes, cryptography mode (encrypt/decrypt)
     * EXPORT: true if process succeed, false otherwise
     * COMMENT: 
     */
    public boolean doCrypto (byte[] password, int mode)
    {
        boolean success = false;
		
        if (fis != null && fos != null) {
            try {
                SecretKey key;
		
                byte[] rawByte, pByte;
                int i = 0, j = 0;
		
                rawByte = new byte[BUFFER];
                pByte = new byte[BUFFER];
                kg.init (new SecureRandom (password));
                key = kg.generateKey ();
                c.init (mode, key);


                while ((i = fis.read (rawByte, 0, rawByte.length)) != -1) {
                    j = c.update (rawByte, 0, i, pByte);
                    fos.write (pByte, 0, j);
                }


                j = c.doFinal (pByte, 0);
                fos.write (pByte, 0, j);
                fos.flush ();       // flush the remainding byte in the buffer
 			
                success = true;

/*
 * NOTE: The buffered I/O works like this. First the data is written to the 
 * buffer, NOT the output file. When, the buffered is full, then the data in 
 * the buffered is written to the output file. Hence, at the final stage 
 * (after the loop), the write() causes new data to be written to the buffer, 
 * thus flushing the previous data (in the buffer) to the output file. At this
 * instant, note the the last piece of data is STILL in the buffer, NOT written
 * to file yet!!! So, the last call of flush() effectively flush the data in
 * the buffer (the number of bytes to be written to the output file is determined
 * by the 'mark' variable) to the output file.
 */

            } catch (Exception e) {
                System.err.println ("ERROR: Incorrect password. Decryption failed...");
                // Assumption: Error/Exception occur is due to incorrect password.
                //e.printStackTrace ();
            } finally {
                try {
                    fis.close ();
                    fos.close ();
                } catch (IOException e) {
                    System.err.println ("ERROR: Unable to releale input-output file stream.");
                }
            }
        }
        return success;
    }
}
