/*
 * FIRA2 is a program to inject/hide a file into a standard bitmap file. 
 * It includes file compression and encryption as well.
 * Copyright (C) 2006 LCC Coprs
 * The author can be reached at aurora.lai@gmail.com
 *
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


/*
 * FILE: FileInjection.java
 * PURPOSE: to create a task for file injection/hiding or file retrieval. 
 * AUTHOR: LCC Corps
 * DATE: 1 December 2005
 * LAST MODIFIED: June 28, 2006
 * REFERENCE:
 * COMMENT: Creation/construction of one Task object can only run either inject/hide file task or
 *          retrieve file task for ONCE. NEVER more than one and NEVER run both tasks for the same
 *          Task object.
 */

package sys;

import java.util.Random;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.File;

public class FileInjection
{
    // the speed approximation is taken to be:
    // normalized speed for 30 kb data is 30kb/95sec
    // the spped is normalize to be achieving 100 when task complete;
    public static final float NORM_INJECT_SPEED = (float)((323.4 * 100)/(30*1024*2)),
                              NORM_RETRIEVE_SPEED = (float) ((1024.0 * 100.0)/(30*1024*2));

    
    private  static final int NO_OF_EOF = 3,     // number of EOF byte used to mark end-of-file
                              PROBABILITY = 4;   // 1 - probability for a file pointer to skip a byte

    private static final long OFFSET = 60;    // first 54-byte of BMP file contains header data, cannot be modified
    private static final byte START_FLAG = -86, FILENAME_FLAG = -94, EOF = -81;
                 // flags used to mark the starting and end point of the
                 // location in the carrier file (with hidden file)

    private MyFile vector, info;		// for file manipulation
    private FileAccess carrier, data;	// for file read & write
  

    // DEFAULT CONSTRUCTOR
    public FileInjection (MyFile data, MyFile carrier)
    {
        super ();
        setDataFile (data);
        setCarrierFile (carrier);
    }

    // CONSTRUCTOR #1
    // PURPOSE: to construct a task with imported attribute
    // IMPORT: carrierBMP pathname, dataFile pathname, an array of password (char)
    public FileInjection (String dataPath, String carrierPath)
    throws FileNotFoundException
    {
        this (new MyFile (dataPath), new MyFile (carrierPath));
    }


    // METHOD: setCarrierFile ()
    // PURPOSE: to set the carrier file
    // IMPORT: a File object
    // EXPORT: none
    public void setCarrierFile (MyFile f)
    {
        if (f != null) {
            try {
                // let the exception handler to catch the file not found error
                carrier = new FileAccess (f , "rws");
                vector = f;
            } catch (FileNotFoundException e) {
                System.err.println ("ERROR: File not found: " +f.getName ());
            }
        }
    }


    // METHOD: setDataFile ()
    // PURPOSE: to set the data file
    // IMPORT: a File object
    // EXPORT: none
    public void setDataFile (MyFile f)
    {
        if (f != null) {
            try {
                // let the exception handler to catch the file not found error
                data = new FileAccess (f, "rwd");
                info = f;
            } catch (FileNotFoundException e) {
                System.err.println ("ERROR: File not found: " +f.getName ());
            }
        }
    }


    // METHOD: doInject()
    // PURPOSE: to execute the whole file injection process
    // IMPORT: password unique ID number
    // EXPORT: true if injection process succedd, false otherwise
    // COMMENT: The reading & writing format for injection process is:
    //             1. Inject a byte FIRST
    //             1. Then, Determine/Get next position
    //          The process is opposite of the retrieval process
    public boolean doInject (long passID)
    {
        Random rand = new Random (passID);
        boolean success = false;
        long pos = getStartPosition (passID);
        byte tempVal = 0;
        byte[] filename = info.truncateName().getBytes();

        if (carrier != null && data != null && canInject ()) {
            // inject start-of-injection flag
            carrier.injectByte (START_FLAG, pos);
            pos = getNextPosition (pos, rand);

            // inject filename
            for (int i = 0; i < filename.length; i++) {
                carrier.injectByte (filename[i], pos);
                pos = getNextPosition (pos, rand);
            }

            // inject end-of-filename flag
            carrier.injectByte (FILENAME_FLAG, pos);
            pos = getNextPosition (pos, rand);

            try {
                // inject file contents
                while (!data.eof ()) {
                    tempVal = data.readByte ();
                    carrier.injectByte (tempVal, pos);
                    pos = getNextPosition (pos, rand);
                }

                // use 3 consecutive EOF byte as end-of-file flag. Use 3 consecutive to avoid/decrease
                // likelihood of false flag when injecting other than text file, e.g. bin or exe file
                for (int i = 0; i < NO_OF_EOF; i++)
                    carrier.injectByte (EOF, pos + i * 8/FileAccess.BIT_PER_BYTE);
          
                success = true;
            } catch (IOException e) {
                System.err.println ("ERROR: Injection process halted at " +pos);
            }
        }
    
        return success;
    }


    // METHOD: doRetrieve()
    // PURPOSE: to execute the whole file retrieval process
    // IMPORT: password unique ID number
    // EXPORT: true if retrieval process succeed, false otherwise
    // COMMENT: In order to skip the START_FLAG, the reading format for retrieval process is:
    //             1. Determine/Get next position FIRST
    //             2. Then, read that byte.
    //          The process is opposite of the injection process.
    //          The termination of file retrieval task is marked by flags.
    public boolean doRetrieve (long passID)
    {
        Random rand = new Random (passID);
        long pos = getStartPosition (passID);
        byte tempVal = 0;
        String filename = "";
        boolean finish = false;
        
        if (carrier != null && hasInject (passID)) {
            // retrieve filename
            pos = getNextPosition (pos, rand);
            tempVal = carrier.retrieveByte (pos);
            while (tempVal != FILENAME_FLAG) {
                filename += (char)tempVal;
                pos = getNextPosition (pos, rand);
                tempVal = carrier.retrieveByte (pos);
            }

            // create 'data' (hidden) file
            if (data == null) {
                // if data file already exists, replace it
                MyFile temp = new MyFile (vector.getParentFile(), filename);
                if (temp.exists ()) {
                    temp.delete ();
                    try {
                        temp.createNewFile ();
                    } catch (IOException e) {
                        System.err.println ("ERROR - Unable to recover data file...");  
                    }
                }
                setDataFile (temp);
            }

            // retrieve the content of the hidden file
            pos = getNextPosition (pos, rand);
            tempVal = carrier.retrieveByte (pos);
            while (!finish) {
                try {
                    data.writeByte (tempVal);
                } catch (IOException e) {
                    System.err.println ("ERROR: Unable to write the retrieved valued to file !!!");
                }

                pos = getNextPosition (pos, rand);
                tempVal = carrier.retrieveByte (pos);

                if (tempVal == EOF) {
                    finish = true;
                    
                    for (int i = 1; i < NO_OF_EOF && finish; i++)
                        finish = finish && carrier.retrieveByte (pos + i * 8/FileAccess.BIT_PER_BYTE) == EOF;
                }
            }
            // delete the START_FLAG once file retrieval is complete
            carrier.injectByte ((byte) 0, getStartPosition (passID));   
        }
    
        return finish;
    }

    
    // METHOD: getStartPos()
    // PURPOSE: to calculate the starting position for injection
    // IMPORT: password unique ID number
    // EXPORT: the start position of this task
    //private
    public long getStartPosition (long id)
    {
        long start = id;

        start %= getMaxLength () * 8/FileAccess.BIT_PER_BYTE;
        start += OFFSET;

        return start;
    }


    // METHOD: getMaxLength()
    // PURPOSE: to calculate the maximum length of the file (calculated in bytes)
    //          that can be safely injected to a bitmap file
    // IMPORT: none
    // EXPORT: the max length of this task
    //private
    public long getMaxLength ()
    {
        long length;

        // The + 2 in the denom is the margin for the randomly skip a bit
        length = (vector.length() - OFFSET) / (8/FileAccess.BIT_PER_BYTE + 2);

        return length;
    }


    // METHOD: canInject ()
    // PURPOSE: to determine if the selected bitmap image file can be injected with a data file
    // IMPORT: none
    // EXPORT: true if the bitmap file can be injected with a file
    public boolean canInject ()
    {
        boolean can = false;
        can = (info.length () < getMaxLength ());

        return can;
    }


    // METHOD: hasInject ()
    // PURPOSE: to determine if the selected bitmap image file has a file injected/hidden within it
    // IMPORT: password unique ID number
    // EXPORT: true if the bitmap file has a hidden file

    public boolean hasInject (long id)
    {
        long pos = getStartPosition (id);
        boolean has = (START_FLAG == carrier.retrieveByte (pos));

        return has;
    }


    // METHOD: getNextPosition ()
    // PURPOSE: to determine and return the next position for file injection
    // IMPORT: the current position and a Random object
    // EXPORT: the next position
    //private
    public long getNextPosition (long currentPos, Random r)
    {
        long nextPos = currentPos + 8/FileAccess.BIT_PER_BYTE + (r.nextInt (10) / PROBABILITY);

        try {
            if (nextPos >= carrier.length () - 8/FileAccess.BIT_PER_BYTE) {
                // first 60-byte of a bitmap file is file header, CANNOT be altered
                nextPos = OFFSET;   
            }
        } catch (IOException e) {
            System.err.println ("Error in determining next position for injection...");
        }

        return nextPos;
    }
  
    /*
     * METHOD: closeFiles()
     * PURPOSE: to close and release all the file I/O stream
     * IMPORT: none
     * EXPORT:none
     * COMMENT:
     */
    public void closeFiles ()
    throws IOException
    {
        if (data != null)
            data.close ();
    
        if (carrier != null)
            carrier.close ();
    }
  
  
    /*
     * METHOD: getDataFile()
     * PURPOSE: to return the data file to the calling function
     * IMPORT: none
     * EXPORT: MyFile data fila
     * COMMENT:
     */
    public MyFile getDataFile ()
    {
        return info;
    }
  
    
    /*
     * METHOD: getCarrierFile()
     * PURPOSE: to return the carrier file to the calling function
     * IMPORT: none
     * EXPORT: MyFile carrier file
     * COMMENT:
     */
    public MyFile getCarrierFile ()
    {
        return vector;
    }
}


