/*
 * FIRA2 is a program to inject/hide a file into a standard bitmap file. 
 * It includes file compression and encryption as well.
 * Copyright (C) 2006 LCC Coprs
 * The author can be reached at aurora.lai@gmail.com
 *
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


/*
 * FILE: Password.java
 * INHERITS: none
 * PURPOSE: to create a password object
 * AUTHOR: LCC Corps
 * DATE: 26 November 2005
 * VERSION: 1.00
 * REFERENCE:
 */


package sys;

import java.util.Random;

public class Password
{
    public static final int MIN_LENGTH = 2,    // minimum length for a valid password
                            MAX_LENGTH = 20;   // maximum length

    private byte password[];
  
    // CONSTRUCTOR #1
    // PURPOSE: to initialize the attributes with desired value
    // IMPORT: a password in array of byte
    // COMMENT: byte[] is chosen to be input param since the SecureRandom object 
    //          in Cipher class require byte[] as parameter. Hence, the returned 
    //          char[] from getPassword() in JPasswordField is converted to 
    //          byte[] externally
    public Password (byte [] pass)
    {
        setPassword (pass);
    }


    // METHOD: setPassword ()
    // PURPOSE: to set the password
    // IMPORT: a password as an array of byte
    // EXPORT: none
    // COMMENT: This method is declared as private for security purpose. 
    //          Password can ONLY be set during object construction
    private void setPassword (byte [] pass)
    {
        if (pass != null)
        password = cloneBytes (pass);
    }


    // METHOD: cloneBytes ()
    // PURPOSE: to clone/copy a byte array
    // IMPORT: a password as an array of byte
    // EXPORT: a copied/cloned byte array
    public static byte[] cloneBytes (byte[] source)
    {
        byte[] target = new byte[source.length];

        for (int i = 0; i < source.length; i++) {
            target[i] = source[i];
        }

        return target;
    }
  
  
    // METHOD: getPassword ()
    // PURPOSE: to obtain the password of this class
    // IMPORT: none
    // EXPORT: a copied/cloned byte array of the password
    public byte[] getPassword ()
    {
        return cloneBytes (password);
    }


    // METHOD: getUniqueID ()
    // PURPOSE: to calculate and return the unique ID for the entered password
    // IMPORT: none
    // EXPORT: the password unique ID
    // COMMENT: Each password entered will result in a unique ID. Same password 
    //          will same UniqueID!!! A call to this method from an INVALID
    //          password object will also return its uniqueID
    public long getUniqueID ()
    {
        Random rand;
        String temp = new String (password);

        long uniqueID = temp.hashCode ();   // get the hash code from the String object
        rand = new Random (uniqueID);  // and use the hash code as Random object seed

        // process to further manipulate password hash code to produce a unique ID
        do {
            uniqueID *= password[rand.nextInt (password.length)];
            uniqueID += password[rand.nextInt (password.length)];
            uniqueID /= password[rand.nextInt (password.length)];
            uniqueID -= password[rand.nextInt (password.length)];
        } while (rand.nextBoolean ());

        uniqueID = Math.abs (uniqueID);   // make sure that the uniqueID is only 
                                          // POSITIVE numbers
 
        return uniqueID;
    }


    // METHOD: isValid ()
    // PURPOSE: to check if a password is valid or not
    // IMPORT: none
    // EXPORT: true if password is valid, false otherwise
    // COMMENT: A valid password must be at least MIN_LENGTH and at most MAX_LENGTH
    public boolean isValid ()
    {
        int len = password.length;

        return (MIN_LENGTH <= len && len <= MAX_LENGTH);
    }
}