/*
 * FIRA2 is a program to inject/hide a file into a standard bitmap file. 
 * It includes file compression and encryption as well.
 * Copyright (C) 2006 LCC Coprs
 * The author can be reached at aurora.lai@gmail.com
 *
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


/* 
 * FILE: Task.java
 * INHERITS: none
 * PURPOSE: To combine all the file compression, file cryptograph, and 
 *          file injection/retrieval processes
 * AUTHOR: LCC Corps
 * DATE: May 16, 2006
 * LAST UPDATED: June 20, 2006
 * REFERENCE:
 * COMMENT:
 *
 */
package sys;

import javax.crypto.Cipher;
import javax.swing.JOptionPane;
import java.io.IOException;
import sys.*;

public class Task extends Thread
{
    public static final int INJECT = 0,
                            RETRIEVE = 1;

    private MyFile data, carrier;
    private Password pass;
    private int mode;
    private boolean startInject,    // to flag the progress bar to go to determinate state
                    isComplete, 
                    hasError;

    // COnstructor #1
    public Task (MyFile data, MyFile carrier, Password pass, int mode)
    {
        super ();
        this.data = data;
        this.carrier = carrier;
        this.pass = pass;
        this.mode = mode;
        startInject = false;
        isComplete = false;
        hasError = false;
        super.setPriority (3);
    }

    
    /*
     * METHOD: run()
     * PURPOSE: to implement the run() for the thread. 
     * IMPORT: none
     * EXPORT: none
     * COMMENT: Note that this run() will only call the appropriate inject or 
     *          retrieve methods depending on the task mode
     */
    public void run ()
    {
        try {
            if (mode == INJECT) {
                inject ();
            } else if (mode == RETRIEVE) {
                retrieve ();
            } else {
                System.err.println ("ERROR: Unknown mode selected...");
            }
        } catch (IOException e) {
            JOptionPane.showMessageDialog (null,
                    "Fatal error occurred. The application will be closed.",
                    "Fatal error.",
                    JOptionPane.ERROR_MESSAGE);
            System.exit (0);
        }
    }

    
    /*
     * METHOD: inject()
     * PURPOSE: to perform the actual file injection process, including the 
     *          compression, and encryption
     * IMPORT: none
     * EXPORT: none
     * COMMENT: to test this method in the unit test case, the access modifier 
     *          for this method need to be changed to public
     */
    public void inject ()
    throws IOException
    {
        MyFile temp;
        FileCompression fcom;
        FileCrypto fcry;
        FileInjection finj;
        
        // create temp file to accomodate compressed data file
        temp = MyFile.createTempFile ("inject", "$zip", null);
        
        // performing data file compression
        fcom = new FileCompression (data, temp);
        fcom.zip ();
        temp = temp.renameTo (data, true);

        data = temp;    // to avoid accidental referencing using temp instead of data
        temp = null;
	fcom = null;
      
        // test to see if can inject file or not i.e. data file size < carrier file size
        finj = new FileInjection (data, carrier);
        if (finj.canInject ()) {
            finj.closeFiles (); // to close and release all file I/O stream
            finj = null;        // reset FileInjection object
            
            // create temp file to accomodate the encrypted data file
            temp = MyFile.createTempFile ("inject", "&enc", null);
            
            // performing the data file encryption
            fcry = new FileCrypto (data, temp);
            fcry.doCrypto (pass.getPassword(), Cipher.ENCRYPT_MODE);
            fcry = null;    // reset FileCrypto object

            /* WHY CANNOT DELETE THE FILE???
             * Because previously, the FileAccess (inherited from RandomAccessFile)
             * in finj (FileInjection object) is not closed, and has their stream
             * released. So the delete() operation fails, because the file is 
             * identified to be still in use by the system!!!
             */
            data.delete();	// delete first temp file, compressed file
            temp = temp.renameTo (data, true);      // optional, just to preserve 
                                                    // original data file name
            data = temp;
            temp = null;

            // performing file injection process
            startInject = true;
            finj = new FileInjection (data, carrier);
            finj.doInject (pass.getUniqueID ());
        } else {
            startInject = true;  // if error occur, assume injection process has started,
            hasError = true;     // but with error
        }
 
        /* 
         * perform some finalizing task, close I/O files and delete temp files created 
         * for data file.
         * NOTE: This block of code is put here to increase code reusability. If the 
         * previous conditional block is true (if part executed, the data file being 
         * deleted is actually the compressed and encrypted version of the original 
         * data file). If, for the previous conditional block, the else part is 
         * executed, then the file being deleted is the compressed (NO ENCRYPTED) 
         * version of the original file
         */
        try {
            finj.closeFiles ();
        } catch (IOException e) {
            System.err.println ("ERROR - Unable to finalize file injection process!");
        }
        finj = null;
        data.delete ();     // delete the compressed and encrypted temp data file
        
        isComplete = true;
    }
	

    /*
     * METHOD: retrieve()
     * PURPOSE: to perform the actual file retrieval process, including the 
     *          decompression, and decryption
     * IMPORT: none
     * EXPORT: none
     * COMMENT: to test this method in the unit test case, the access modifier 
     *          for this method need to be changed to public
     */
    public void retrieve ()
    throws IOException
    {
        FileInjection finj;
        FileCrypto fcry;
        FileCompression fcom;
        MyFile temp;

        // performing file retrieval process
        finj = new FileInjection (data, carrier);

        // if file retrieval finish successfully, continue with remaining tasks
        if (finj.doRetrieve (pass.getUniqueID ())) {
            data = finj.getDataFile ();     // return the data file recovered from the retrieval process
            finj.closeFiles ();
            finj = null;

            // create temp file to accomodate the retrieved raw file
            temp = MyFile.createTempFile ("retrieve", "$dec", null);
            
            // performing data file decryption
            fcry = new FileCrypto (data, temp);
            fcry.doCrypto (pass.getPassword (), Cipher.DECRYPT_MODE);
            
            data.delete ();
            temp = temp.renameTo (data, true);
            data = temp;
            temp = null;

            // Unzip the file to the same location as the carrier file
            fcom = new FileCompression (data, carrier.getParentFile ());
            fcom.unzip ();
            
            data.delete ();
        } else {
            finj.closeFiles ();
            hasError = true;
        }
        
        isComplete = true;
    }
	

    /*
     * METHOD: hasStarted()
     * PURPOSE: to return if the file injection process (after file compression 
     *          and encryption) has started
     * IMPORT: none
     * EXPORT: flag to indicate file injection has started or not
     * COMMENT: 
     */
    public boolean hasStarted ()
    {
        return startInject;
    }
    
    
    /*
     * METHOD: hasError()
     * PURPOSE: to return if the is error in performing all the task
     * IMPORT: none
     * EXPORT: flag to indicate if error exists
     * COMMENT: 
     */
    public boolean hasError ()
    {
        return hasError;
    }
    
    
    /*
     * METHOD: isComplete()
     * PURPOSE: to return if the task has completed or not
     * IMPORT: none
     * EXPORT: flag to indicate task completion
     * COMMENT: 
     */
    public boolean isComplete ()
    {
        return isComplete;
    }
    
    
    /*
     * METHOD: getDataFile()
     * PURPOSE: to return the data file for this task
     * IMPORT: none
     * EXPORT: MyFile data file
     * COMMENT: 
     */
    public MyFile getDataFile ()
    {
        return data;
    }
    
    
    /*
     * METHOD: getCarrierFile()
     * PURPOSE: to return the carrier file for this task
     * IMPORT: none
     * EXPORT: MyFile carrier file
     * COMMENT: 
     */
    public MyFile getCarrierFile ()
    {
        return carrier;
    }

    
    // For debugging
    // to print a string
    public void print (String a)
    {
        System.out.println (a);
    }
}